#!/bin/bash
########################################################################
#
# build_ninja.sh <product_dir> [tar] [source]
#
# NOTE: Ninja itself does not require python to operate, so to avoid a
# dependence upon a particular python we do *not* install any auxiliary
# files that would. Since these are only useful for a mode of use of
# ninja that we do not support (i.e. hand-written ninja code), we do not
# believe this to be a problem.
#
# CHG 2015/03/19.
########################################################################
usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
      flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}
    etcdir=${product_dir}/${package}/${pkgver}/etc
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}
    tardir=${product_dir}/${package}/${pkgver}/tar
    patchdir=${product_dir}/${package}/${pkgver}/patch

}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}
sourcetar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=ninja
pkgver=v1_5_3
python_version=v2_7_9 # Needed for build only.
ssibuildshims_version=v0_08_04
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=v${pkgdotver}.tar.gz

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

get_platform_name

if [ "${maketar}" = "tar" ] && [ -x "${pkgdir}/bin/ninja" ]
then
    ups exist ${package} ${pkgver} || ${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_QUAL
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball -d etc ${product_dir} ${package} ${pkgver} NO_QUAL
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

cd ${blddir}
tar xf ${tardir}/${pkgtarfile} || die "ERROR: untar source failed."
cd ${package}-${pkgdotver} || die "ERROR: unable to change to package build directory."

IFS=. system_python_version=($(python -V 2>&1 | cut -d' ' -f2))
if (( ${system_python_version[0]} < 2 )) || \
   { (( ${system_python_version[0]} == 2 )) && \
     (( ${system_python_version[1]} < 6 )); }; then # Too old.
  setup python ${python_version} || \
    die "ERROR: unable to set up python ${python_version} in order to build ninja."
  python=`which python`
else # System python should be fine.
  python=/usr/bin/python
fi

echo "INFO: Using ${python} for build only."
set -x
"${python}" ./configure.py --with-python="${python}" --bootstrap || \
  die "ERROR: unable to build ninja."

install -d ${etcdir} || die "ERROR: unable to ensure existence of ${etcdir}/"
install -m 0644 misc/{{bash,zsh}-completion,ninja-mode.el,ninja.vim} \
  ${etcdir}/ || \
  die "ERROR: unable to install miscellaneous files in ${etcdir}"
install -d "${pkgdir}/bin" || \
  die "ERROR: unable to ensure existence of ${pkgdir}/bin"
install ninja "${pkgdir}/bin/" || \
  die "ERROR: unable to install ninja binary in ${pkgdir/bin}"
set +x

[[ -x ${pkgdir}/bin/ninja ]] || \
  die "ERROR: failed to create ${pkgdir}/bin/ninja\n       Something is very wrong."

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_QUAL

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver}
echo "${package} is installed at ${NINJA_DIR}"

# this must be last because it will exit
if [ "${maketar}" = "tar" ] && [ -x "${pkgdir}/bin/ninja" ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball -d etc ${product_dir} ${package} ${pkgver} NO_QUAL
fi

exit 0
